@@ -40,29 +40,37 @@
 #include "gtkhtml-types.h"
 #include "gtkhtml.h"
 
-#define GTK_HTML_PROPERTY(w,p)         (GTK_HTML_CLASS (GTK_OBJECT (w)->klass)->properties-> ## p)
+#define GTK_HTML_PROPERTY(w,p)         (GTK_HTML_CLASS (GTK_OBJECT (w)->klass)->properties-> p)
 
 struct _GtkHTMLClassProperties {
 	/* viewer */
 	gboolean  animations;
-	gchar    *font_var_family;
+	gchar    *font_var;
 	guint     font_var_size;
-	gchar    *font_fix_family;
+	gboolean  font_var_points;
+	gchar    *font_fix;
 	guint     font_fix_size;
-	gchar    *font_var_family_print;
+	gboolean  font_fix_points;
+	gchar    *font_var_print;
 	guint     font_var_size_print;
-	gchar    *font_fix_family_print;
+	gboolean  font_var_print_points;
+	gchar    *font_fix_print;
 	guint     font_fix_size_print;
+	gboolean  font_fix_print_points;
 
 	/* editor */
 	gboolean  magic_links;
+	gboolean  magic_smileys;
 	gchar    *keybindings_theme;
 
 	gboolean  live_spell_check;
 	GdkColor  spell_error_color;
 	gchar    *language;
-};
 
+	gchar    *link_color;
+	gchar    *vlink_color;
+	gchar    *alink_color;
+};
 
 GtkHTMLClassProperties * gtk_html_class_properties_new       (void);
 void                     gtk_html_class_properties_destroy   (GtkHTMLClassProperties *p);
