@@ -41,93 +41,28 @@
 
 HTMLFrameClass html_frame_class;
 static HTMLEmbeddedClass *parent_class = NULL;
-static gboolean calc_size (HTMLObject *o, HTMLPainter *painter);
+static gboolean calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs);
 
 static void
 frame_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
 {
 	HTMLFrame *frame = HTML_FRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
+
+	gtk_signal_emit_by_name (GTK_OBJECT (parent->engine), "url_requested",
+				 url, handle);
+}
+
+static void
+frame_set_base (GtkHTML *html, const gchar *url, gpointer data)
+{
 	char *new_url = NULL;
-	
-	/* FIXME this is not exactly the single safest method of expanding a relative url */
-	if (frame->url && !strstr (url, ":")) {
-		char *base = g_strdup (frame->url);
-		if (*url == '/' && strstr (base, ":")) {
-			int i = 0;
-			char *cur = base; 
-			
-			while (*cur != '\0') {
-				if (*cur == '/') i++;
-				
-				if (i == 3) {
-					*(++cur) = '\0';
-					break;
-				}
-				cur++;
-			}
-			new_url = g_strconcat (base, "/", url, NULL);
-		} else if (*url == '/' && (*base == '/' || *base == '\0')) {
-			new_url = NULL;
-		} else {
-			new_url = g_strconcat (base, "/", url, NULL);
-		}		
-		g_free (base);
-	}
-	g_warning ("url = %s, frame->url = %s, new_url = %s", url, frame->url, new_url);
-	
-	gtk_signal_emit_by_name (GTK_OBJECT (parent->engine), "url_requested", new_url ? new_url : url,
-				 handle);
-	
+
+	new_url = gtk_html_get_url_base_relative (html, url);
+	gtk_html_set_base (html, new_url);
 	g_free (new_url);
 }
 
-static void
-frame_set_base (GtkHTML *html, const gchar *base, gpointer data)
-{
-	HTMLFrame *frame = HTML_FRAME (data);
-	char *new_url = NULL;
-	char *cur;
-	
-	/* FIXME this is not exactly the single safest method of expanding a relative url */
-	if (frame->url && !strstr (base, ":"))
-		new_url = g_strconcat (frame->url, "/", base, NULL);
-	else 
-		new_url = g_strdup (base);
-
-	cur = new_url + strlen (new_url);
-	while (cur >= new_url) {
-		if (*cur == '/') {
-			break;
-		}
-		cur--;
-	}	
-	*(cur + 1)= '\0';
-			
-	g_warning ("base = %s, frame->url = %s, new_url = %s", base, frame->url, new_url);
-
-	g_free (frame->url);
-	frame->url = new_url;
-}
-
-static void
-frame_on_url (GtkHTML *html, const gchar *url, gpointer data)
-{
-	HTMLFrame *frame = HTML_FRAME (data);
-	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
-
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "on_url", url);
-}
-
-static void
-frame_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
-{
-	HTMLFrame *frame = HTML_FRAME (data);
-	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(frame)->parent);
-
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "link_clicked", url);
-}
-
 static void
 frame_submit (GtkHTML *html, 
 	      const gchar *method, 
@@ -307,8 +242,7 @@ check_page_split (HTMLObject *self, gint y)
 }
 
 static gboolean
-calc_size (HTMLObject *o,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLFrame *frame;
 	HTMLEngine *e;
@@ -324,7 +258,7 @@ calc_size (HTMLObject *o,
 
 	if ((frame->width < 0) && (frame->height < 0)) {
 		e->width = o->max_width;
-		html_engine_calc_size (e);
+		html_engine_calc_size (e, changed_objs);
 
 		height = html_engine_get_doc_height (e);
 		width = html_engine_get_doc_width (e);
@@ -338,8 +272,7 @@ calc_size (HTMLObject *o,
 		o->ascent = height;
 		o->descent = 0;
 	} else
-		return (* HTML_OBJECT_CLASS (parent_class)->calc_size) 
-			(o, painter);
+		return (* HTML_OBJECT_CLASS (parent_class)->calc_size) (o, painter, changed_objs);
 
 	if (o->descent != old_descent
 	    || o->ascent != old_ascent
@@ -435,6 +368,17 @@ append_selection_string (HTMLObject *self,
 	html_object_append_selection_string (GTK_HTML (HTML_FRAME (self)->html)->engine->clue, buffer);
 }
 
+static void
+reparent (HTMLEmbedded *emb, GtkWidget *html)
+{
+	HTMLFrame *frame = HTML_FRAME (emb);
+
+	gtk_html_set_iframe_parent (GTK_HTML (frame->html), 
+				    html,
+				    GTK_HTML (frame->html)->frame);
+	(* HTML_EMBEDDED_CLASS (parent_class)->reparent) (emb, html);
+}
+
 static gboolean
 select_range (HTMLObject *self,
 	      HTMLEngine *engine,
@@ -542,12 +486,12 @@ html_frame_init (HTMLFrame *frame,
 					border ? GTK_SHADOW_IN : GTK_SHADOW_NONE);
 
 #endif
-	  
-	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
 
 	new_widget = gtk_html_new ();
 	new_html = GTK_HTML (new_widget);
 
+	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
+
 	html_engine_set_tokenizer (new_html->engine, new_tokenizer);
 	gtk_object_unref (GTK_OBJECT (new_tokenizer));
 	new_tokenizer = NULL;
@@ -559,32 +503,33 @@ html_frame_init (HTMLFrame *frame,
 	gtk_container_add (GTK_CONTAINER (scrolled_window), new_widget);
 	gtk_widget_show (new_widget);
 
+	frame->url = g_strdup (src);
 	frame->width = width;
 	frame->height = height;
 	frame->gdk_painter = NULL;
-	frame->url = NULL;
+	gtk_html_set_base (new_html, src);
 
-	if (src && strcmp (src, "")) {
-		handle = gtk_html_begin (new_html);
-		gtk_signal_emit_by_name (GTK_OBJECT (new_html->engine), 
-					 "url_requested", src, handle);
+	handle = gtk_html_begin (new_html);
+	gtk_html_set_base (new_html, src);
 
-		frame_set_base (new_html, src, frame);
-	} else {		
-		gtk_html_load_empty (new_html);
-	}
 	new_html->engine->clue->parent = HTML_OBJECT (frame);
 
 
 	gtk_signal_connect (GTK_OBJECT (new_html), "url_requested",
 			    GTK_SIGNAL_FUNC (frame_url_requested),
 			    (gpointer)frame);
+#if 0
+	/* NOTE: because of peculiarities of the frame/gtkhtml relationship
+	 * on_url and link_clicked are emitted from the toplevel widget not
+	 * proxied like url_requested is.
+	 */
 	gtk_signal_connect (GTK_OBJECT (new_html), "on_url",
 			    GTK_SIGNAL_FUNC (frame_on_url), 
 			    (gpointer)frame);
 	gtk_signal_connect (GTK_OBJECT (new_html), "link_clicked",
 			    GTK_SIGNAL_FUNC (frame_link_clicked),
 			    (gpointer)frame);	
+#endif
 	gtk_signal_connect (GTK_OBJECT (new_html), "size_changed",
 			    GTK_SIGNAL_FUNC (frame_size_changed),
 			    (gpointer)frame);	
@@ -594,17 +539,21 @@ html_frame_init (HTMLFrame *frame,
 	gtk_signal_connect (GTK_OBJECT (new_html), "submit",
 			    GTK_SIGNAL_FUNC (frame_submit),
 			    (gpointer)frame);
+	gtk_signal_connect (GTK_OBJECT (new_html), "set_base",
+			    GTK_SIGNAL_FUNC (frame_set_base), 
+			    (gpointer)frame);
 
 	html_frame_set_margin_height (frame, 0);
 	html_frame_set_margin_width (frame, 0);
-	/*
-	  gtk_signal_connect (GTK_OBJECT (new_html), "set_base",
-	  GTK_SIGNAL_FUNC (frame_set_base), (gpointer)frame);
 
+	/*
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (frame_button_press_event), frame);
 	*/
 
+	gtk_signal_emit_by_name (GTK_OBJECT (parent_html->engine), 
+				 "url_requested", src, handle);
+
 	gtk_widget_set_usize (scrolled_window, width, height);
 
 	gtk_widget_show (scrolled_window);	
@@ -621,6 +570,7 @@ html_frame_init (HTMLFrame *frame,
 				     parent_html->engine->settings->color_set);
 	html_colorset_set_unchanged (new_html->engine->settings->color_set,
 				     parent_html->engine->settings->color_set);
+	html_painter_set_focus (new_html->engine->painter, parent_html->engine->have_focus);
 	/*
 	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
 			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
@@ -673,6 +623,8 @@ html_frame_class_init (HTMLFrameClass *klass,
 	object_class->draw_background         = draw_background;
 	object_class->append_selection_string = append_selection_string;
 	object_class->select_range            = select_range;
+
+	embedded_class->reparent = reparent;
 }
 
 
