@@ -1,4 +1,3 @@
-
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* This file is part of the GtkHTML library.
    
@@ -34,6 +33,7 @@
 #include "htmlselection.h"
 #include "htmlsettings.h"
 #include "htmltokenizer.h"
+#include "htmlembedded.h"
 
 #ifndef USE_SCROLLED_WINDOW
 #include <gal/widgets/e-scroll-frame.h>
@@ -42,41 +42,24 @@
 HTMLIFrameClass html_iframe_class;
 static HTMLEmbeddedClass *parent_class = NULL;
 static gboolean calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs);
+	
+static void
+iframe_set_base (GtkHTML *html, const char *url, gpointer data)
+{
+	char *new_url = gtk_html_get_url_base_relative (html, url);
+
+	gtk_html_set_base (html, new_url);
+	g_free (new_url);
+}
 
 static void
 iframe_url_requested (GtkHTML *html, const char *url, GtkHTMLStream *handle, gpointer data)
 {
 	HTMLIFrame *iframe = HTML_IFRAME (data);
 	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
-	char *new_url = NULL;
 
-	/* FIXME this is not exactly the single safest method of expanding a relative url */
-	if (!strstr (url, ":"))
-		new_url = g_strconcat (iframe->url, url, NULL);
-	
-	gtk_signal_emit_by_name (GTK_OBJECT (parent->engine), "url_requested", new_url ? new_url : url,
-				 handle);
-	
-	if (new_url)
-		g_free (new_url);
-}
-
-static void
-iframe_on_url (GtkHTML *html, const gchar *url, gpointer data)
-{
-	HTMLIFrame *iframe = HTML_IFRAME (data);
-	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
-
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "on_url", url);
-}
-
-static void
-iframe_link_clicked (GtkHTML *html, const gchar *url, gpointer data)
-{
-	HTMLIFrame *iframe = HTML_IFRAME (data);
-	GtkHTML *parent = GTK_HTML (HTML_EMBEDDED(iframe)->parent);
-
-	gtk_signal_emit_by_name (GTK_OBJECT (parent), "link_clicked", url);
+	gtk_signal_emit_by_name (GTK_OBJECT (parent->engine), "url_requested",
+				 url, handle);
 }
 
 static void
@@ -245,10 +228,19 @@ copy (HTMLObject *self,
         HTMLIFrame *d = HTML_IFRAME (dest);
 
 	(* HTML_OBJECT_CLASS (parent_class)->copy) (self, dest);
-
+	/*
+	html_iframe_init (s, &html_iframe_class,
+			  HTML_EMBEDDED (dest)->parent,
+			  s->url,
+			  s->width,
+			  s->height,
+			  s->frameborder);
+	*/
 	d->scroll = NULL;
-	d->html = NULL;
+	d->html = gtk_html_new ();
 	d->gdk_painter = NULL;
+	d->old_painter = NULL;
+	d->parent_painter = NULL;
 
 	d->url = g_strdup (s->url);
 	d->width = s->width;
@@ -256,6 +248,24 @@ copy (HTMLObject *self,
 	d->frameborder = s->frameborder;
 }
 
+static HTMLObject *
+op_copy (HTMLObject *self, HTMLObject *parent, HTMLEngine *e, GList *from, GList *to, guint *len)
+{
+	HTMLObject *dup, *clue;
+	GtkHTML *html;
+
+	dup = html_object_dup (self);
+	html = HTML_IFRAME (dup)->html;
+	clue = GTK_HTML (HTML_IFRAME (self)->html)->engine->clue;
+	GTK_HTML (HTML_IFRAME (dup)->html)->engine->clue =
+		html_object_op_copy (clue, dup, GTK_HTML (HTML_IFRAME (self)->html)->engine,
+				     html_object_get_bound_list (clue, from),
+				     html_object_get_bound_list (clue, to), len);
+	GTK_HTML (HTML_IFRAME (dup)->html)->engine->clue->parent = parent;
+	
+	return dup;
+}
+
 void
 html_iframe_set_scrolling (HTMLIFrame *iframe, GtkPolicyType scroll)
 {
@@ -368,8 +378,8 @@ check_point (HTMLObject *self,
 	    || y >= self->y + self->descent || y < self->y - self->ascent)
 		return NULL;
 
-	x -= self->x + e->leftBorder;
-	y -= self->y - self->ascent + e->topBorder;
+	x -= self->x + e->leftBorder - e->x_offset;
+	y -= self->y - self->ascent + e->topBorder - e->y_offset;
 
 	if (for_cursor && (x < 0 || y < e->clue->y - e->clue->ascent))
 		return html_object_check_point (e->clue, e->painter, 0, e->clue->y - e->clue->ascent,
@@ -395,6 +405,17 @@ append_selection_string (HTMLObject *self,
 	html_object_append_selection_string (GTK_HTML (HTML_IFRAME (self)->html)->engine->clue, buffer);
 }
 
+static void
+reparent (HTMLEmbedded *emb, GtkWidget *html)
+{
+	HTMLIFrame *iframe = HTML_IFRAME (emb);
+
+	gtk_html_set_iframe_parent (GTK_HTML (iframe->html), 
+				    html,
+				    GTK_HTML (iframe->html)->frame);
+	(* HTML_EMBEDDED_CLASS (parent_class)->reparent) (emb, html);
+}
+
 /* static gboolean
 select_range (HTMLObject *self,
 	      HTMLEngine *engine,
@@ -417,6 +438,12 @@ destroy (HTMLObject *o)
 	g_free (iframe->url);
 
 	if (iframe->html) {
+		if (iframe->old_painter)
+			gtk_object_unref (GTK_OBJECT (iframe->old_painter));
+		if (iframe->parent_painter)
+			gtk_object_unref (GTK_OBJECT (iframe->parent_painter));
+		iframe->old_painter = iframe->parent_painter = NULL;
+
 		gtk_signal_disconnect_by_data (GTK_OBJECT (iframe->html), o);
 		iframe->html = NULL;
 	}
@@ -462,6 +489,12 @@ html_iframe_init (HTMLIFrame *iframe,
 	new_widget = gtk_html_new ();
 	new_html = GTK_HTML (new_widget);
 
+	/* use parent_html font_manager */
+	gtk_object_ref (GTK_OBJECT (new_html->engine->painter));
+	gtk_object_ref (GTK_OBJECT (parent_html->engine->painter));
+	iframe->old_painter = new_html->engine->painter;
+	iframe->parent_painter = parent_html->engine->painter;
+
 	new_tokenizer = html_tokenizer_clone (parent_html->engine->ht);
 
 	html_engine_set_tokenizer (new_html->engine, new_tokenizer);
@@ -479,30 +512,43 @@ html_iframe_init (HTMLIFrame *iframe,
 	iframe->width = width;
 	iframe->height = height;
 	iframe->gdk_painter = NULL;
+	gtk_html_set_base (new_html, src);
 
 	handle = gtk_html_begin (new_html);
+
 	new_html->engine->clue->parent = HTML_OBJECT (iframe);
 
 	gtk_signal_connect (GTK_OBJECT (new_html), "url_requested",
 			    GTK_SIGNAL_FUNC (iframe_url_requested),
 			    (gpointer)iframe);
+#if 0
+	/* NOTE: because of peculiarities of the frame/gtkhtml relationship
+	 * on_url and link_clicked are emitted from the toplevel widget not
+	 * proxied like url_requested is.
+	 */
 	gtk_signal_connect (GTK_OBJECT (new_html), "on_url",
 			    GTK_SIGNAL_FUNC (iframe_on_url), 
 			    (gpointer)iframe);
 	gtk_signal_connect (GTK_OBJECT (new_html), "link_clicked",
 			    GTK_SIGNAL_FUNC (iframe_link_clicked),
 			    (gpointer)iframe);	
+#endif 
 	gtk_signal_connect (GTK_OBJECT (new_html), "size_changed",
 			    GTK_SIGNAL_FUNC (iframe_size_changed),
 			    (gpointer)iframe);	
+	gtk_signal_connect (GTK_OBJECT (new_html), "set_base",
+			    GTK_SIGNAL_FUNC (iframe_set_base),
+			    (gpointer)iframe);	
 	gtk_signal_connect (GTK_OBJECT (new_html), "object_requested",
 			    GTK_SIGNAL_FUNC (iframe_object_requested),
 			    (gpointer)iframe);	
+
 	/*
 	  gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 	  GTK_SIGNAL_FUNC (iframe_button_press_event), iframe);
 	*/
-	gtk_signal_emit_by_name (GTK_OBJECT (new_html->engine), 
+
+	gtk_signal_emit_by_name (GTK_OBJECT (parent_html->engine), 
 				 "url_requested", src, handle);
 
 	gtk_widget_set_usize (scrolled_window, width, height);
@@ -523,14 +569,10 @@ html_iframe_init (HTMLIFrame *iframe,
 	/*
 	gtk_signal_connect (GTK_OBJECT (html), "title_changed",
 			    GTK_SIGNAL_FUNC (title_changed_cb), (gpointer)app);
-	gtk_signal_connect (GTK_OBJECT (html), "set_base",
-			    GTK_SIGNAL_FUNC (on_set_base), (gpointer)app);
 	gtk_signal_connect (GTK_OBJECT (html), "button_press_event",
 			    GTK_SIGNAL_FUNC (on_button_press_event), popup_menu);
 	gtk_signal_connect (GTK_OBJECT (html), "redirect",
 			    GTK_SIGNAL_FUNC (on_redirect), NULL);
-	gtk_signal_connect (GTK_OBJECT (html), "submit",
-			    GTK_SIGNAL_FUNC (on_submit), NULL);
 	gtk_signal_connect (GTK_OBJECT (html), "object_requested",
 			    GTK_SIGNAL_FUNC (object_requested_cmd), NULL);
 	*/
@@ -565,6 +607,7 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->reset                   = reset;
 	object_class->draw                    = draw;
 	object_class->copy                    = copy;
+	object_class->op_copy                 = op_copy;
 	object_class->set_max_width           = set_max_width;
 	object_class->forall                  = forall;
 	object_class->check_page_split        = check_page_split;
@@ -576,4 +619,6 @@ html_iframe_class_init (HTMLIFrameClass *klass,
 	object_class->is_container            = is_container;
 	object_class->draw_background         = draw_background;
 	object_class->append_selection_string = append_selection_string;
+	
+	embedded_class->reparent = reparent;
 }
