@@ -29,6 +29,7 @@
 #include "htmlengine-edit.h"
 #include "htmlengine-save.h"
 #include "htmlpainter.h"
+#include "htmlplainpainter.h"
 #include "htmltable.h"
 #include "htmltablepriv.h"
 #include "htmltablecell.h"
@@ -72,16 +73,18 @@ draw_background_helper (HTMLTableCell *cell,
 		}
 	} else if (t && t->bgPixmap)
 		pixbuf = t->bgPixmap->pixbuf;
-
-	html_painter_draw_background (p,
-				      color,
-				      pixbuf,
-				      tx + paint->x0,
-				      ty + paint->y0,
-				      paint->x1 - paint->x0,
-				      paint->y1 - paint->y0,
-				      paint->x0 - o->x,
-				      paint->y0 - (o->y - o->ascent));
+	
+	/* FIXME this should be moved into the painter interface */
+	if (!HTML_IS_PLAIN_PAINTER (p))
+		html_painter_draw_background (p,
+					      color,
+					      pixbuf,
+					      tx + paint->x0,
+					      ty + paint->y0,
+					      paint->x1 - paint->x0,
+					      paint->y1 - paint->y0,
+					      paint->x0 - o->x,
+					      paint->y0 - (o->y - o->ascent));
 }
 
 
@@ -108,16 +111,16 @@ copy (HTMLObject *self, HTMLObject *dest)
 }
 
 static gboolean
-merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *right)
+merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList **left, GList **right, HTMLCursor *cursor)
 {
 	HTMLTableCell *c1 = HTML_TABLE_CELL (self);
 	HTMLTableCell *c2 = HTML_TABLE_CELL (with);
 
-	g_print ("merge cells %d,%d %d,%d\n", c1->row, c1->col, c2->row, c2->col);
+	/* g_print ("merge cells %d,%d %d,%d\n", c1->row, c1->col, c2->row, c2->col); */
 
-	if (HTML_OBJECT_TYPE (with) == HTML_TYPE_CLUEV || c1->col == c2->col) {
+	if (HTML_OBJECT_TYPE (with) == HTML_TYPE_CLUEV || (c1->col == c2->col && c1->row == c2->row)) {
 		gboolean rv;
-		rv = (* HTML_OBJECT_CLASS (parent_class)->merge) (self, with, e, left, right);
+		rv = (* HTML_OBJECT_CLASS (parent_class)->merge) (self, with, e, left, right, cursor);
 		if (rv && with->parent && HTML_IS_TABLE (with->parent)) {
 			self->next = NULL;
 			html_object_remove_child (with->parent, with);
@@ -126,6 +129,7 @@ merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GList *ri
 					     HTML_TABLE_CELL (self)->row, HTML_TABLE_CELL (self)->col,
 					     HTML_TABLE_CELL (self));
 		}
+
 		return rv;
 	} else
 		return FALSE;
@@ -186,13 +190,17 @@ clue_move_children (HTMLClue *clue, gint x_delta, gint y_delta)
 }
 
 static gboolean
-calc_size (HTMLObject *o, HTMLPainter *painter)
+calc_size (HTMLObject *o, HTMLPainter *painter, GList **changed_objs)
 {
 	HTMLTableCell *cell;
 	gboolean rv;
+	gint old_width, old_height;
+
+	old_width  = o->width;
+	old_height = o->ascent + o->descent;
 
 	cell = HTML_TABLE_CELL (o);
-	rv   = (* HTML_OBJECT_CLASS (parent_class)->calc_size) (o, painter);
+	rv   = (* HTML_OBJECT_CLASS (parent_class)->calc_size) (o, painter, changed_objs);
 
 	if (cell->fixed_height && o->ascent + o->descent < cell->fixed_height) {
 		gint remains = cell->fixed_height - (o->ascent + o->descent);
@@ -215,6 +223,9 @@ calc_size (HTMLObject *o, HTMLPainter *painter)
 		rv = TRUE;
 	}
 
+	if (o->parent && (o->width != old_width || o->ascent + o->descent != old_height))
+		html_object_add_to_changed (changed_objs, o->parent);
+
 	return rv;
 }
 
